package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.ASN1Object;
import cn.com.infosec.asn1.ASN1OctetString;
import cn.com.infosec.asn1.ASN1Sequence;
import cn.com.infosec.asn1.DERBitString;
import cn.com.infosec.asn1.DERNull;
import cn.com.infosec.asn1.DERObject;
import cn.com.infosec.asn1.DERObjectIdentifier;
import cn.com.infosec.asn1.DEROctetString;
import cn.com.infosec.asn1.cryptopro.CryptoProObjectIdentifiers;
import cn.com.infosec.asn1.cryptopro.ECGOST3410NamedCurves;
import cn.com.infosec.asn1.cryptopro.GOST3410PublicKeyAlgParameters;
import cn.com.infosec.asn1.x509.AlgorithmIdentifier;
import cn.com.infosec.asn1.x509.SubjectPublicKeyInfo;
import cn.com.infosec.asn1.x9.X962Parameters;
import cn.com.infosec.asn1.x9.X9ECParameters;
import cn.com.infosec.asn1.x9.X9ECPoint;
import cn.com.infosec.asn1.x9.X9IntegerConverter;
import cn.com.infosec.asn1.x9.X9ObjectIdentifiers;
import cn.com.infosec.crypto.params.ECDomainParameters;
import cn.com.infosec.crypto.params.ECPublicKeyParameters;
import cn.com.infosec.jce.ECGOST3410NamedCurveTable;
import cn.com.infosec.jce.interfaces.ECPointEncoder;
import cn.com.infosec.jce.spec.ECNamedCurveParameterSpec;
import cn.com.infosec.jce.spec.ECNamedCurveSpec;
import cn.com.infosec.math.ec.ECCurve;
import cn.com.infosec.math.ec.ECFieldElement;
import cn.com.infosec.math.ec.ECPoint;
import java.io.IOException;
import java.math.BigInteger;
import java.security.interfaces.ECPublicKey;
import java.security.spec.ECParameterSpec;
import java.security.spec.ECPublicKeySpec;
import java.security.spec.EllipticCurve;

public class JCEECPublicKey implements ECPublicKey, cn.com.infosec.jce.interfaces.ECPublicKey, ECPointEncoder {
   private String algorithm;
   private ECPoint q;
   private ECParameterSpec ecSpec;
   private boolean withCompression;
   private GOST3410PublicKeyAlgParameters gostParams;

   JCEECPublicKey(String algorithm, JCEECPublicKey key) {
      this.algorithm = "EC";
      this.algorithm = algorithm;
      this.q = key.q;
      this.ecSpec = key.ecSpec;
      this.withCompression = key.withCompression;
      this.gostParams = key.gostParams;
   }

   JCEECPublicKey(String algorithm, ECPublicKeySpec spec) {
      this.algorithm = "EC";
      this.algorithm = algorithm;
      ECParameterSpec algorithm1;
      ECParameterSpec var10001 = algorithm1 = spec.getParams();
      this.ecSpec = algorithm1;
      this.q = EC5Util.convertPoint(var10001, spec.getW(), false);
   }

   JCEECPublicKey(String algorithm, cn.com.infosec.jce.spec.ECPublicKeySpec spec) {
      this.algorithm = "EC";
      this.algorithm = algorithm;
      this.q = spec.getQ();
      if (spec.getParams() != null) {
         this.ecSpec = EC5Util.convertSpec(EC5Util.convertCurve(spec.getParams().getCurve(), spec.getParams().getSeed()), spec.getParams());
      } else {
         if (this.q.getCurve() == null) {
            this.q = ProviderUtil.getEcImplicitlyCa().getCurve().createPoint(this.q.getX().toBigInteger(), this.q.getY().toBigInteger(), false);
         }

         this.ecSpec = null;
      }

   }

   JCEECPublicKey(String algorithm, ECPublicKeyParameters params, ECParameterSpec spec) {
      String var10004 = algorithm;
      super();
      this.algorithm = "EC";
      ECDomainParameters algorithm1 = params.getParameters();
      this.algorithm = var10004;
      this.q = params.getQ();
      if (spec == null) {
         this.ecSpec = this.createSpec(EC5Util.convertCurve(algorithm1.getCurve(), algorithm1.getSeed()), algorithm1);
      } else {
         this.ecSpec = spec;
      }

   }

   JCEECPublicKey(String algorithm, ECPublicKeyParameters params, cn.com.infosec.jce.spec.ECParameterSpec spec) {
      String var10004 = algorithm;
      super();
      this.algorithm = "EC";
      ECDomainParameters algorithm1 = params.getParameters();
      this.algorithm = var10004;
      this.q = params.getQ();
      if (spec == null) {
         this.ecSpec = this.createSpec(EC5Util.convertCurve(algorithm1.getCurve(), algorithm1.getSeed()), algorithm1);
      } else {
         this.ecSpec = EC5Util.convertSpec(EC5Util.convertCurve(spec.getCurve(), spec.getSeed()), spec);
      }

   }

   JCEECPublicKey(String algorithm, ECPublicKeyParameters params) {
      this.algorithm = "EC";
      this.algorithm = algorithm;
      this.q = params.getQ();
      this.ecSpec = null;
   }

   private ECParameterSpec createSpec(EllipticCurve ellipticCurve, ECDomainParameters dp) {
      EllipticCurve var10002 = ellipticCurve;
      java.security.spec.ECPoint var3;
      var3 = new java.security.spec.ECPoint.<init>(dp.getG().getX().toBigInteger(), dp.getG().getY().toBigInteger());
      BigInteger ellipticCurve1 = dp.getN();
      int dp1 = dp.getH().intValue();
      return new ECParameterSpec(var10002, var3, ellipticCurve1, dp1);
   }

   JCEECPublicKey(ECPublicKey key) {
      this.algorithm = "EC";
      this.algorithm = key.getAlgorithm();
      ECParameterSpec var2;
      ECParameterSpec var10001 = var2 = key.getParams();
      this.ecSpec = var2;
      this.q = EC5Util.convertPoint(var10001, key.getW(), false);
   }

   JCEECPublicKey(SubjectPublicKeyInfo info) {
      this.algorithm = "EC";
      int var5;
      ASN1OctetString var10000;
      EllipticCurve var16;
      if (info.getAlgorithmId().getObjectId().equals(CryptoProObjectIdentifiers.gostR3410_2001)) {
         DERBitString var2 = info.getPublicKeyData();
         this.algorithm = "ECGOST3410";

         try {
            var10000 = (ASN1OctetString)ASN1Object.fromByteArray(var2.getBytes());
         } catch (IOException var9) {
            throw new IllegalArgumentException("error recovering public key");
         }

         byte[] var14 = var10000.getOctets();
         int var3 = 32;
         byte[] var4 = new byte[32];
         var5 = 32;
         byte[] var6 = new byte[32];

         for(int var7 = 0; var7 != var3; ++var7) {
            var4[var7] = var14[31 - var7];
         }

         for(var3 = 0; var3 != var5; ++var3) {
            var6[var3] = var14[63 - var3];
         }

         GOST3410PublicKeyAlgParameters var15;
         GOST3410PublicKeyAlgParameters var10003 = var15 = new GOST3410PublicKeyAlgParameters;
         var10003.<init>((ASN1Sequence)info.getAlgorithmId().getParameters());
         this.gostParams = var10003;
         ECNamedCurveParameterSpec info1;
         ECCurve var10002 = (info1 = ECGOST3410NamedCurveTable.getParameterSpec(ECGOST3410NamedCurves.getName(var15.getPublicKeyParamSet()))).getCurve();
         var16 = EC5Util.convertCurve(var10002, info1.getSeed());
         this.q = var10002.createPoint(new BigInteger(1, var4), new BigInteger(1, var6), false);
         String var10 = ECGOST3410NamedCurves.getName(this.gostParams.getPublicKeyParamSet());
         java.security.spec.ECPoint var20;
         var20 = new java.security.spec.ECPoint.<init>(info1.getG().getX().toBigInteger(), info1.getG().getY().toBigInteger());
         BigInteger info2 = info1.getN();
         BigInteger var22 = info1.getH();
         this.ecSpec = new ECNamedCurveSpec(var10, var16, var20, info2, var22);
      } else {
         X962Parameters var17;
         X962Parameters var31 = var17 = new X962Parameters;
         var31.<init>((DERObject)info.getAlgorithmId().getParameters());
         X9ECParameters var21;
         ECCurve var23;
         BigInteger var24;
         if (var31.isNamedCurve()) {
            DERObjectIdentifier var18;
            EllipticCurve var27 = EC5Util.convertCurve(var23 = (var21 = ECUtil.getNamedCurveByOid(var18 = (DERObjectIdentifier)var17.getParameters())).getCurve(), var21.getSeed());
            String var19 = ECUtil.getCurveName(var18);
            java.security.spec.ECPoint var28;
            var28 = new java.security.spec.ECPoint.<init>(var21.getG().getX().toBigInteger(), var21.getG().getY().toBigInteger());
            var24 = var21.getN();
            BigInteger var29 = var21.getH();
            this.ecSpec = new ECNamedCurveSpec(var19, var27, var28, var24, var29);
         } else if (var17.isImplicitlyCA()) {
            this.ecSpec = null;
            var23 = ProviderUtil.getEcImplicitlyCa().getCurve();
         } else {
            X9ECParameters var10001 = var21 = new X9ECParameters;
            var10001.<init>((ASN1Sequence)var17.getParameters());
            var16 = EC5Util.convertCurve(var23 = var10001.getCurve(), var21.getSeed());
            java.security.spec.ECPoint var25;
            var25 = new java.security.spec.ECPoint.<init>(var21.getG().getX().toBigInteger(), var21.getG().getY().toBigInteger());
            var24 = var21.getN();
            var5 = var21.getH().intValue();
            this.ecSpec = new ECParameterSpec(var16, var25, var24, var5);
         }

         byte[] info3;
         byte[] var32 = info3 = info.getPublicKeyData().getBytes();
         Object var26;
         var26 = new DEROctetString.<init>(info3);
         if (var32[0] == 4 && info3[1] == info3.length - 2 && (info3[2] == 2 || info3[2] == 3) && (new X9IntegerConverter()).getByteLength(var23) >= info3.length - 3) {
            try {
               var10000 = (ASN1OctetString)ASN1Object.fromByteArray(info3);
            } catch (IOException var8) {
               throw new IllegalArgumentException("error recovering public key");
            }

            var26 = var10000;
         }

         this.q = (new X9ECPoint(var23, (ASN1OctetString)var26)).getPoint();
      }

   }

   private void extractBytes(byte[] encKey, int offSet, BigInteger bI) {
      int var4;
      byte[] var6;
      if ((var6 = bI.toByteArray()).length < 32) {
         byte[] bI1 = new byte[32];
         var4 = 32 - var6.length;
         int var5 = var6.length;
         System.arraycopy(var6, 0, bI1, var4, var5);
      }

      for(int bI2 = 0; bI2 != 32; ++bI2) {
         var4 = offSet + bI2;
         encKey[var4] = var6[var6.length - 1 - bI2];
      }

   }

   public String getAlgorithm() {
      return this.algorithm;
   }

   public String getFormat() {
      return "X.509";
   }

   public byte[] getEncoded() {
      BigInteger var2;
      BigInteger var6;
      DERObjectIdentifier var10;
      SubjectPublicKeyInfo var12;
      SubjectPublicKeyInfo var23;
      if (this.algorithm.equals("ECGOST3410")) {
         GOST3410PublicKeyAlgParameters var1;
         if ((var1 = this.gostParams) == null) {
            var1 = new GOST3410PublicKeyAlgParameters.<init>(ECGOST3410NamedCurves.getOID(((ECNamedCurveSpec)this.ecSpec).getName()), CryptoProObjectIdentifiers.gostR3411_94_CryptoProParamSet);
         }

         JCEECPublicKey var10000 = this;
         JCEECPublicKey var10001 = this;
         JCEECPublicKey var10002 = this;
         var6 = this.q.getX().toBigInteger();
         var2 = var10002.q.getY().toBigInteger();
         byte[] var3;
         var10001.extractBytes(var3 = new byte[64], 0, var6);
         var10000.extractBytes(var3, 32, var2);
         SubjectPublicKeyInfo var7;
         var23 = var7 = new SubjectPublicKeyInfo;
         GOST3410PublicKeyAlgParameters var10003 = var1;
         var10 = CryptoProObjectIdentifiers.gostR3410_2001;
         AlgorithmIdentifier var25 = new AlgorithmIdentifier(var10, var10003.getDERObject());
         DEROctetString var11;
         var11 = new DEROctetString.<init>(var3);
         var23.<init>(var25, var11);
         var12 = var7;
      } else {
         ECParameterSpec var13;
         X962Parameters var14;
         X962Parameters var15;
         if ((var13 = this.ecSpec) instanceof ECNamedCurveSpec) {
            var10 = ECUtil.getNamedCurveOid(((ECNamedCurveSpec)var13).getName());
            var14 = new X962Parameters.<init>(var10);
            var15 = var14;
         } else if (var13 == null) {
            var14 = new X962Parameters.<init>(DERNull.INSTANCE);
            var15 = var14;
         } else {
            ECCurve var20 = EC5Util.convertCurve(var13.getCurve());
            X9ECParameters var16;
            X9ECParameters var24 = var16 = new X9ECParameters;
            ECCurve var26 = var20;
            ECCurve var10005 = var20;
            java.security.spec.ECPoint var21 = this.ecSpec.getGenerator();
            ECPoint var22 = EC5Util.convertPoint(var10005, var21, this.withCompression);
            BigInteger var17 = this.ecSpec.getOrder();
            BigInteger var4 = BigInteger.valueOf((long)this.ecSpec.getCofactor());
            byte[] var5 = this.ecSpec.getCurve().getSeed();
            var24.<init>(var26, var22, var17, var4, var5);
            var15 = new X962Parameters.<init>(var16);
         }

         ECCurve var18 = this.engineGetQ().getCurve();
         ECCurve var28 = var18;
         JCEECPublicKey var30 = this;
         JCEECPublicKey var10004 = this;
         var6 = this.getQ().getX().toBigInteger();
         var2 = var10004.getQ().getY().toBigInteger();
         boolean var19 = var30.withCompression;
         ASN1OctetString var8 = (ASN1OctetString)(new X9ECPoint(var28.createPoint(var6, var2, var19))).getDERObject();
         var23 = var12 = new SubjectPublicKeyInfo;
         ASN1OctetString var27 = var8;
         AlgorithmIdentifier var9;
         AlgorithmIdentifier var29 = var9 = new AlgorithmIdentifier;
         X962Parameters var31 = var15;
         var10 = X9ObjectIdentifiers.id_ecPublicKey;
         var29.<init>(var10, var31.getDERObject());
         var23.<init>(var9, var27.getOctets());
      }

      return var12.getDEREncoded();
   }

   public ECParameterSpec getParams() {
      return this.ecSpec;
   }

   public cn.com.infosec.jce.spec.ECParameterSpec getParameters() {
      ECParameterSpec var1;
      return (var1 = this.ecSpec) == null ? null : EC5Util.convertSpec(var1, this.withCompression);
   }

   public java.security.spec.ECPoint getW() {
      return new java.security.spec.ECPoint(this.q.getX().toBigInteger(), this.q.getY().toBigInteger());
   }

   public ECPoint getQ() {
      if (this.ecSpec == null) {
         ECFieldElement var1;
         JCEECPublicKey var10002;
         ECFieldElement var2;
         if (this.q instanceof ECPoint.Fp) {
            var10002 = this;
            var2 = this.q.getX();
            var1 = var10002.q.getY();
            return new ECPoint.Fp((ECCurve)null, var2, var1);
         } else {
            var10002 = this;
            var2 = this.q.getX();
            var1 = var10002.q.getY();
            return new ECPoint.F2m((ECCurve)null, var2, var1);
         }
      } else {
         return this.q;
      }
   }

   ECPoint engineGetQ() {
      return this.q;
   }

   cn.com.infosec.jce.spec.ECParameterSpec engineGetSpec() {
      ECParameterSpec var1;
      return (var1 = this.ecSpec) != null ? EC5Util.convertSpec(var1, this.withCompression) : ProviderUtil.getEcImplicitlyCa();
   }

   public String toString() {
      StringBuffer var10000 = new StringBuffer();
      String var1 = System.getProperty("line.separator");
      var10000.append("EC Public Key").append(var1);
      var10000.append("            X: ").append(this.q.getX().toBigInteger().toString(16)).append(var1);
      var10000.append("            Y: ").append(this.q.getY().toBigInteger().toString(16)).append(var1);
      return var10000.toString();
   }

   public void setPointFormat(String style) {
      this.withCompression = "UNCOMPRESSED".equalsIgnoreCase(style) ^ true;
   }

   public boolean equals(Object o) {
      if (!(o instanceof JCEECPublicKey)) {
         return false;
      } else {
         JCEECPublicKey o1 = (JCEECPublicKey)o;
         return this.engineGetQ().equals(o1.engineGetQ()) && this.engineGetSpec().equals(o1.engineGetSpec());
      }
   }

   public int hashCode() {
      return this.engineGetQ().hashCode() ^ this.engineGetSpec().hashCode();
   }
}
